@@ -135,7 +135,7 @@ char * modestring(enum mad_mode mode)
     switch(mode)
     {
         case MAD_MODE_SINGLE_CHANNEL:
-            return "single-channel";
+            return "mono";
         case MAD_MODE_DUAL_CHANNEL:
             return "dual-channel";
         case MAD_MODE_JOINT_STEREO:
@@ -209,6 +209,7 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
         status = MPG321_STOPPED;
         return MAD_FLOW_STOP;
     }
+
     if(options.opt & MPG321_VERBOSE_PLAY || options.opt & MPG321_REMOTE_PLAY)
     {
         mad_timer_string(current_time, long_currenttime_str, "%.2u:%.2u.%.2u", MAD_UNITS_MINUTES,
@@ -231,7 +232,6 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
     /* update cached table of frames & times */
     if (current_frame <= playbuf->num_frames) /* we only allocate enough for our estimate. */
     {
-
         playbuf->frames[current_frame] = playbuf->frames[current_frame-1] + (header->bitrate / 8 / 1000)
             * mad_timer_count(header->duration, MAD_UNITS_MILLISECONDS);
         playbuf->times[current_frame] = current_time;
@@ -255,6 +255,7 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
                 header->flags & MAD_FLAG_PROTECTION ? 1 : 0, header->emphasis,
                 header->bitrate/1000, header->mode_extension);
         }    
+
         else if (options.opt & MPG321_VERBOSE_PLAY)/*zip it good*/
         {
             fprintf(stderr, "MPEG 1.0, Layer: %s, Freq: %d, mode: %s, modext: %d, BPF : %ld\n"
@@ -267,7 +268,8 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
                     header->flags & MAD_FLAG_ORIGINAL ? "Yes" : "No", header->flags & MAD_FLAG_PROTECTION ? "Yes" : "No",
                     header->emphasis, header->bitrate/1000, header->mode_extension,
                     header->samplerate, MAD_NCHANNELS(header));
-            }
+        }
+
         else if (!(options.opt & MPG321_QUIET_PLAY))/*I love Joey*/
         {
             fprintf(stderr, "MPEG 1.0 layer %s, %ld kbit/s, %d Hz %s\n",
@@ -277,15 +279,19 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
     
     if (options.opt & MPG321_VERBOSE_PLAY)
     {
-        fprintf(stderr, "Frame# %5lu [%5lu], Time: %s [%s], \r", current_frame, 
-                playbuf->num_frames - current_frame, long_currenttime_str, long_remaintime_str);
+        if (!options.skip_printing_frames 
+            || (options.skip_printing_frames && !(current_frame % options.skip_printing_frames)))
+            fprintf(stderr, "Frame# %5lu [%5lu], Time: %s [%s], \r", current_frame, 
+                    playbuf->num_frames - current_frame, long_currenttime_str, long_remaintime_str);
     }
     
     else if (options.opt & MPG321_REMOTE_PLAY)
     {
-        printf("@F %ld %ld %.2f %.2f\n", current_frame, playbuf->num_frames - current_frame,
-            ((double)mad_timer_count(current_time, MAD_UNITS_CENTISECONDS)/100.0),
-            ((double)mad_timer_count(time_remaining, MAD_UNITS_CENTISECONDS)/100.0));
+        if (!options.skip_printing_frames 
+            || (options.skip_printing_frames && !(current_frame % options.skip_printing_frames)))
+            printf("@F %ld %ld %.2f %.2f\n", current_frame, playbuf->num_frames - current_frame,
+                ((double)mad_timer_count(current_time, MAD_UNITS_CENTISECONDS)/100.0),
+                ((double)mad_timer_count(time_remaining, MAD_UNITS_CENTISECONDS)/100.0));
     }
     
     return MAD_FLOW_CONTINUE;
@@ -554,44 +560,47 @@ enum mad_flow move(buffer *buf, signed long frames)
     
 int calc_length(char *file, buffer *buf)
 {
-    FILE * f;
+    int f;
     struct stat filestat;
     void *fdm;
     char buffer[3];
 
-    if (stat(file, &filestat) == -1)
+    f = open(file, O_RDONLY);
+
+    if (f < 0)
     {
         mpg321_error(file);
         return -1;
     }
 
-    f = fopen(file, "r");
-
-    if (f == NULL)
+    if (fstat(f, &filestat) < 0)
     {
         mpg321_error(file);
+        close(f);
         return -1;
     }
 
     if (!S_ISREG(filestat.st_mode))
     {
         fprintf(stderr, "%s: Not a regular file\n", file);
-        fclose(f);
+        close(f);
         return -1;
     }
 
     /* TAG checking is adapted from XMMS */
     buf->length = filestat.st_size;
 
-    if (fseek(f, -128, SEEK_END) < 0)
+    if (lseek(f, -128, SEEK_END) < 0)
     {
         mpg321_error(file);
+        close(f);
         return -1;
     }    
 
-    if (read(fileno(f), buffer, 3) != 3)
+    if (read(f, buffer, 3) != 3)
     {
         fprintf(stderr, "Couldnt read 3 bytes from %s\n", file);
+        close(f);
         return -1;
     }
     
@@ -600,12 +609,11 @@ int calc_length(char *file, buffer *buf)
         buf->length -= 128; /* Correct for id3 tags */
     }
     
-    rewind(f);
-
-    fdm = mmap(0, buf->length, PROT_READ, MAP_SHARED, fileno(f), 0);
+    fdm = mmap(0, buf->length, PROT_READ, MAP_SHARED, f, 0);
     if (fdm == MAP_FAILED)
     {
         mpg321_error(file);
+        close(f);
         return -1;
     }
 
@@ -616,11 +624,11 @@ int calc_length(char *file, buffer *buf)
     if (munmap(fdm, buf->length) == -1)
     {
         mpg321_error(file);
-        fclose(f);
+        close(f);
         return -1;
     }
 
-    if (fclose(f) == EOF)
+    if (close(f) < 0)
     {
         mpg321_error(file);
         return -1;
@@ -629,28 +637,82 @@ int calc_length(char *file, buffer *buf)
     return 0;
 }
 
-/* Scale and round samples to 16 bits. Taken directly from minimad.c,
-   part of the libmad distribution. */
-static inline 
-signed int scale(mad_fixed_t sample)
+/* The following two routines and data structure are from the ever-brilliant
+   Rob Leslie.
+*/
+
+struct audio_dither {
+  mad_fixed_t error[3];
+  mad_fixed_t random;
+};
+
+/*
+* NAME:        prng()
+* DESCRIPTION: 32-bit pseudo-random number generator
+*/
+static inline
+unsigned long prng(unsigned long state)
 {
-  /* round */
-  sample += (1L << (MAD_F_FRACBITS - 16));
-
-  /* clip */
-  if (sample >= MAD_F_ONE)
-    sample = MAD_F_ONE - 1;
-  else if (sample < -MAD_F_ONE)
-    sample = -MAD_F_ONE; 
-
-  /* quantize */
-  return sample >> (MAD_F_FRACBITS + 1 - 16);
+  return (state * 0x0019660dL + 0x3c6ef35fL) & 0xffffffffL;
 }
 
-/* Borrowed from SDL. There, the maximum volume is 128, which makes sense given that the
-   sample is 8 bits. */
-   
-#define ADJUST_VOLUME(s,v)  ((s)*(v)/128)
+/*
+* NAME:        audio_linear_dither()
+* DESCRIPTION: generic linear sample quantize and dither routine
+*/
+inline
+signed long audio_linear_dither(unsigned int bits, mad_fixed_t sample,
+                                struct audio_dither *dither)
+{
+  unsigned int scalebits;
+  mad_fixed_t output, mask, random;
+
+  enum {
+    MIN = -MAD_F_ONE,
+    MAX =  MAD_F_ONE - 1
+  };
+
+  /* noise shape */
+  sample += dither->error[0] - dither->error[1] + dither->error[2];
+
+  dither->error[2] = dither->error[1];
+  dither->error[1] = dither->error[0] / 2;
+
+  /* bias */
+  output = sample + (1L << (MAD_F_FRACBITS + 1 - bits - 1));
+
+  scalebits = MAD_F_FRACBITS + 1 - bits;
+  mask = (1L << scalebits) - 1;
+
+  /* dither */
+  random  = prng(dither->random);
+  output += (random & mask) - (dither->random & mask);
+
+  dither->random = random;
+
+  /* clip */
+  if (output > MAX) {
+    output = MAX;
+
+    if (sample > MAX)
+      sample = MAX;
+  }
+  else if (output < MIN) {
+    output = MIN;
+
+    if (sample < MIN)
+      sample = MIN;
+  }
+
+  /* quantize */
+  output &= ~mask;
+
+  /* error feedback */
+  dither->error[0] = sample - output;
+
+  /* scale */
+  return output >> scalebits;
+}
 
 enum mad_flow output(void *data,
                      struct mad_header const *header,
@@ -661,21 +723,37 @@ enum mad_flow output(void *data,
     
     static unsigned char stream[1152*4]; /* 1152 because that's what mad has as a max; *4 because
                                     there are 4 distinct bytes per sample (in 2 channel case) */
+    static unsigned int rate = 0;
+    static int channels = 0;
+    static struct audio_dither dither;
+
     register char * ptr = stream;
     register signed int sample;
+    register mad_fixed_t tempsample;
 
     /* We need to know information about the file before we can open the playdevice
        in some cases. So, we do it here. */
     if (!playdevice)
     {
+        channels = MAD_NCHANNELS(header);
+        rate = header->samplerate;
         open_ao_playdevice(header);        
     }
-    
+
+    else if ((channels != MAD_NCHANNELS(header) || rate != header->samplerate) && playdevice_is_live())
+    {
+        ao_close(playdevice);
+        channels = MAD_NCHANNELS(header);
+        rate = header->samplerate;
+        open_ao_playdevice(header);        
+    }        
+
     if (pcm->channels == 2)
     {
         while (nsamples--)
         {
-            sample = ADJUST_VOLUME(scale(*left_ch++), options.volume);
+            tempsample = (mad_fixed_t)((*left_ch++ * (double)options.volume)/MAD_F_ONE);
+            sample = (signed int) audio_linear_dither(16, tempsample, &dither);
 
 #ifndef WORDS_BIGENDIAN
             *ptr++ = (unsigned char) (sample >> 0);
@@ -685,7 +763,8 @@ enum mad_flow output(void *data,
             *ptr++ = (unsigned char) (sample >> 0);
 #endif
             
-            sample = ADJUST_VOLUME(scale(*right_ch++), options.volume);
+            tempsample = (mad_fixed_t)((*right_ch++ * (double)options.volume)/MAD_F_ONE);
+            sample = (signed int) audio_linear_dither(16, tempsample, &dither);
 #ifndef WORDS_BIGENDIAN
             *ptr++ = (unsigned char) (sample >> 0);
             *ptr++ = (unsigned char) (sample >> 8);
@@ -702,7 +781,8 @@ enum mad_flow output(void *data,
     {
         while (nsamples--)
         {
-            sample = ADJUST_VOLUME(scale(*left_ch++), options.volume);
+            tempsample = (mad_fixed_t)((*left_ch++ * (double)options.volume)/MAD_F_ONE);
+            sample = (signed int) audio_linear_dither(16, tempsample, &dither);
             
 #ifndef WORDS_BIGENDIAN
             *ptr++ = (unsigned char) (sample >> 0);
@@ -716,18 +796,4 @@ enum mad_flow output(void *data,
     }
 
     return MAD_FLOW_CONTINUE;        
-}
-
-#if 0
-char *format_time_short(double t)
-{
-    int minutes, seconds;
-    char * time;
-    
-    minutes = (int)(t / 60);
-    seconds = (int)t-(60 * minutes);
-
-    asprintf(&time,"%.2d:%.2d",minutes, (seconds>0)? seconds: 0);
-    return time;
-}
-#endif
+}
\ No newline at end of file
