@@ -49,7 +49,7 @@ int quit_now = 0;
 char *playlist_file;
 ao_device *playdevice=NULL;
 mad_timer_t current_time;
-mpg321_options options = { 0, NULL, 0 , 0};
+mpg321_options options = { 0, NULL, 0 , 0, 0};
 int status = MPG321_STOPPED;
 
 void mpg123_boilerplate()
@@ -130,7 +130,6 @@ RETSIGTYPE handle_signals(int sig)
 int main(int argc, char *argv[])
 {
     int fd = 0;
-    int already_playing = 0;
     char *currentfile, old_dir[PATH_MAX];
     playlist *pl = NULL;
 
@@ -148,13 +147,13 @@ int main(int argc, char *argv[])
         exit(1);
     }
 
-    /* Get the command line options */
-    options.volume = 100;
+    options.volume = MAD_F_ONE;
 
+    /* Get the command line options */
     parse_options(argc, argv, pl);
 
-    /* If there were no arguments just print the usage */
-    if (optind == argc)
+    /* If there were no files and no playlist specified just print the usage */
+    if (!playlist_file && optind == argc)
     {
         usage(argv[0]);
         exit(0);
@@ -291,17 +290,14 @@ int main(int argc, char *argv[])
                 strncpy(old_dir, dirn, PATH_MAX);
                 old_dir[PATH_MAX-1] = '\0';
             }
-
-            /* Print a newline only between new songs */
-            if (already_playing)
+            
+            /* print a newline between different songs only, not after
+               Directory: listing */
+            else
             {
                 fprintf(stderr, "\n");
             }
-            else
-            {
-                already_playing = 1;
-            }
-            
+
             fprintf(stderr,"Playing MPEG stream from %s ...\n", basen);
     
             free(dirc);
@@ -313,6 +309,11 @@ int main(int argc, char *argv[])
             char * basec = strdup(currentfile);
             char * basen = basename(basec);
             
+            char * dot = strrchr(basen, '.');
+            
+            if (dot)
+                *dot = '\0';
+            
             if (strncmp(old_dir, basen, PATH_MAX) != 0)
             {
                 printf("@I %s\n", basen);
