@@ -28,9 +28,9 @@
 #include "mpg321.h"
 #include "getopt.h"
 
-#include <sys/types.h>
-#include <unistd.h>
+#include <sys/time.h>
 #include <sys/resource.h>
+#include <unistd.h>
 #include <string.h>
 
 void parse_options(int argc, char *argv[], playlist *pl)
@@ -92,6 +92,7 @@ void parse_options(int argc, char *argv[], playlist *pl)
         { "remote", 0, 0, 'R' },
             
         /* takes parameters */
+        { "skip-printing-frames", 1, 0, 'G' },
         { "output", 1, 0, 'o' },
         { "list", 1, 0, '@' },
         { "skip", 1, 0, 'k' },
@@ -103,7 +104,7 @@ void parse_options(int argc, char *argv[], playlist *pl)
     int option_index = 0, c;
 
     while ((c = getopt_long(argc, argv, 
-                                "OPLTNEI824cy01mCun:d:h:f:b:p:r:" /* unimplemented */
+                                "OPLTNEI824cy01mCun:d:h:f:b:p:r:G:" /* unimplemented */
                                 "A:D:vqtsVHzZRo:@:k:w:a:g:",     /* implemented */
                         long_options, &option_index)) != -1)
     {            
@@ -231,17 +232,21 @@ void parse_options(int argc, char *argv[], playlist *pl)
             
             case 'g':
                 options.volume = atoi(optarg);
+                options.volume = (options.volume/100.0) * MAD_F_ONE;
                 break;
 
             case 'I':
                 /* We only try to get high priority. If it fails (just as
                    mpg123 does when run as a user), that's fine */
                 setpriority(PRIO_PROCESS, getpid(), -20);
-                break;            
+                break;
+
+            case 'G':
+                options.skip_printing_frames = atoi(optarg);
+                break;
 
             case 'V':
-                printf("mpg321 version " VERSION "\n"
-                       "Copyright (C) 2001 Joe Drew.\n"
+                printf("mpg321 version " VERSION ". Copyright (C) 2001 Joe Drew.\n\n"
                        "This program is free software; you can redistribute it and/or modify\n"
                        "it under the terms of the GNU General Public License as published by\n"
                        "the Free Software Foundation; either version 2 of the License, or\n"
