@@ -35,6 +35,7 @@
 #include <time.h>
 #include <ctype.h>
 #include <errno.h>
+#include <libgen.h>
 
 #include "mpg321.h"
 
@@ -48,6 +49,7 @@ playlist * new_playlist()
     pl->files_size = DEFAULT_PLAYLIST_SIZE;
     pl->numfiles = 0;
     pl->random_play = 0;
+    strcpy(pl->remote_file, "");
     
     return pl;
 }
@@ -183,6 +185,8 @@ void load_playlist(playlist *pl, char *filename)
 {
     FILE *plist;
     char playlist_buf[PATH_MAX];
+    char *directory;
+    int dirlen;
 
     if (strncmp(filename, "-", 1) == 0)
     {
@@ -191,51 +195,42 @@ void load_playlist(playlist *pl, char *filename)
 
     else if(!(plist = fopen(filename,"r")))
     {
-        fprintf(stderr, "%s: %s\n",filename, strerror(errno));
+        mpg321_error(filename);
         exit(1);
     }
 
-    /* ugh! O(n^2)! */
+    directory = strdup(filename);
+    directory = dirname(directory);
+
+    dirlen = strlen(directory);
+
     while (fgets(playlist_buf, PATH_MAX, plist) != NULL)
     {
         trim_whitespace(playlist_buf);
-                
+
+        if (strlen(playlist_buf) == 0)
+            continue;                
+
         if (pl->numfiles == pl->files_size)
             resize_playlist(pl);
 
-        pl->files[pl->numfiles++] = strdup(playlist_buf);
+        if(playlist_buf[0] != '/') /* relative path */
+        {
+            /* +2 because of null terminating character plus / */
+            pl->files[pl->numfiles] = malloc(strlen(playlist_buf) + dirlen + 2);
+            strcpy(pl->files[pl->numfiles], directory);
+            strcat(pl->files[pl->numfiles], "/");
+            strcat(pl->files[pl->numfiles], playlist_buf);
+        }
+
+        else /* absolute path */
+        {
+            pl->files[pl->numfiles] = strdup(playlist_buf);
+        }
+
+        pl->numfiles++;
     }
     
     if (plist != stdin)
         fclose(plist);
 }
-
-#if 0
-    ofs = 0;
-    while(1){
-        numread = fread(playlist_buf + ofs,1,1024,plist);
-        if(!numread || numread == -1) break;
-        ofs += numread;
-        playlist_buf = realloc(playlist_buf,ofs + 1024);
-    }
-    fclose(plist);
-    p = playlist_buf;
-    while((p = strstr(p,"\n"))){
-        playlist_numfiles ++;
-        p ++;
-    }
-
-    files = (char **)calloc(playlist_numfiles+1,sizeof(char *)); /* +1 for the NULL stringarr terminator */
-    copied = 0;
-
-    if(playlist && playlist_buf){
-        p = playlist_buf;
-        files[copied++] = p;
-        while((p = strstr(p,"\n"))){
-            *p = '\0';
-            p ++;
-            if(*p) files[copied++] = p;
-	    }
-    }
-#endif
-
